import { useMutation, useQuery } from "@apollo/client";
import {
  IconButton,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
} from "@mui/material";
import React, { FC } from "react";
import { Article } from "../generated/graphql";
import EditIcon from "@mui/icons-material/Edit";
import { ARTICLES, REMOVE_ARTICLE } from "./articles.constants";
import { Delete } from "@mui/icons-material";
import { format } from "date-fns";
import { useNavigate } from "react-router-dom";

export const ArticleIndex: FC = () => {
  const { data } = useQuery<{
    articles: Article[];
  }>(ARTICLES, {});

  const [removeArticle] = useMutation<any, { id: string }>(REMOVE_ARTICLE);

  const navigate = useNavigate();

  return (
    <section>
      <TableContainer component={Paper}>
        <Table aria-label="articles table">
          <TableHead>
            <TableRow>
              <TableCell>Article Title</TableCell>
              <TableCell>Published At</TableCell>
              <TableCell>Views</TableCell>
              <TableCell>Comments</TableCell>
              <TableCell>Status</TableCell>
              <TableCell>Actions</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {data?.articles.map((article) => (
              <TableRow key={article.id}>
                <TableCell component="th" scope="row">
                  {article.title}
                </TableCell>
                <TableCell>
                  {article.publishedAt
                    ? format(article.publishedAt, "yyyy-MM-dd HH:mm:ss")
                    : "--"}
                </TableCell>
                <TableCell align="right"> -- </TableCell>
                <TableCell align="right"> -- </TableCell>
                <TableCell align="center">{"Published"}</TableCell>
                <TableCell>
                  <IconButton
                    aria-label="edit"
                    onClick={() => navigate(`/articles/${article.id}`)}
                    size="large"
                  >
                    <EditIcon />
                  </IconButton>
                  <IconButton
                    aria-label="delete"
                    onClick={() =>
                      removeArticle({
                        variables: article,
                      })
                    }
                    size="large"
                  >
                    <Delete />
                  </IconButton>
                </TableCell>
              </TableRow>
            ))}
          </TableBody>
        </Table>
      </TableContainer>
    </section>
  );
};
