import { useMutation, useQuery } from "@apollo/client";
import {
  IconButton,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
} from "@mui/material";
import { FC } from "react";
import { Tag } from "../generated/graphql";
import EditIcon from "@mui/icons-material/Edit";
import { Delete } from "@mui/icons-material";
import { REMOVE_TAG, TAGS } from "./tags.constants";
import { useNavigate } from "react-router-dom";

export const TagsIndex: FC = () => {
  const { data } = useQuery<{
    tags: Tag[];
  }>(TAGS, {});

  const [removeArticle] = useMutation<any, { id: string }>(REMOVE_TAG);

  const navigate = useNavigate();

  return (
    <section>
      <TableContainer component={Paper}>
        <Table aria-label="articles table">
          <TableHead>
            <TableRow>
              <TableCell>Tag Name</TableCell>
              <TableCell>Views</TableCell>
              <TableCell>Articles</TableCell>
              <TableCell>Actions</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {data?.tags.map((tag) => (
              <TableRow key={tag.id}>
                <TableCell component="th" scope="row">
                  {tag.name}
                </TableCell>
                <TableCell align="right"> -- </TableCell>
                <TableCell align="right"> -- </TableCell>
                <TableCell>
                  <IconButton
                    aria-label="edit"
                    onClick={() => navigate(`/tags/${tag.id}/modify`)}
                    size="large"
                  >
                    <EditIcon />
                  </IconButton>
                  <IconButton
                    aria-label="delete"
                    onClick={() =>
                      removeArticle({
                        variables: tag,
                      })
                    }
                    size="large"
                  >
                    <Delete />
                  </IconButton>
                </TableCell>
              </TableRow>
            ))}
          </TableBody>
        </Table>
      </TableContainer>
    </section>
  );
};
